`default_nettype none
`timescale 1ns/1ps

module test_mem_read ();

// テスト対象の出力データ型
typedef logic [9:0] data_t;

// テスト対象の信号
logic   clock;      // クロック入力
logic   reset;      // Active Highの同期リセット入力
logic   out_valid;  // 出力VALID
data_t  out_data;   // 出力データ

// クロック生成 (10ns周期)
initial begin
        clock = 0;
end 
always #(5) begin
    clock = ~clock;
end

// テスト対象のインスタンス化
mem_read dut (.*);

localparam int TEST_COUNTS = 65536; 

initial begin
    $dumpfile("test-mem_read.vcd"); // VCDファイルを出力する
    $dumpvars(0);                    // 全変数をVCDに出力する
    
    // リセットをアサートして戻す
    reset <= 1;
    repeat(2) @(posedge clock);
    reset <= 0;
    @(posedge clock);
    while(!out_valid) @(posedge clock); // 最初のout_validアサートまで待つ

    fork
        begin
            int phase_counter;
            phase_counter = 0;
            // 65536回分の出力を確認する
            for(int i = 0; i < TEST_COUNTS; i++) begin
                for(int phase = 0; phase < 3; phase++) begin
                    bit expected_valid;
                    bit [7:0] expected_data;

                    expected_valid = phase <= 1;    // phase 0, 1のときのみout_validがアサートされる
                    expected_data  = (phase == 0) ? i[7:0] : (phase == 1) ? i[15:8] : 0;    // phase 0のときは下位バイト、phase 1のときは上位バイト、phase 2のときは無視
                    // 期待値チェック
                    if( out_valid !== expected_valid )                 $error("#%04d out_valid mismatch. expected: %d, actual: %d", i, expected_valid, out_valid );
                    if( expected_valid && out_data !== expected_data ) $error("#%04d data mismatch.      expected: %04d, actual: %04d", i, expected_data, out_data);
                    @(posedge clock);
                end
            end
            // 直後10サイクルはout_validがアサートされないことを確認
            for(int i = 0; i < 10; i++) begin
                // 期待値チェック
                if( out_valid ) $error("out_valid must be deasserted after 1024 cycles." );
                @(posedge clock);
            end
        end
        begin
            // タイムアウト検出。余裕を見て 65536 * 3 + 20 サイクル以内でテストが完了することを期待する。
            static int limit = TEST_COUNTS * 3 + 20;
            for(int cycles = 0; cycles < limit; cycles++ ) @(posedge clock);
            $error("Error: simulation timed out after %d cycles.", limit);
        end
    join_any
    disable fork;
    $finish;
end

endmodule

`default_nettype wire